/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/
/*
** 
** Author: Didier Lemoine		Created: 20-07-93 
** 
** Module: dbindex 
** 
** Purpose: 	
** This file contains the routines which provide index management facilities
** for the DB Module
** 
** Module Prefix: DB_ 
** 
** Last Update:		hewett
** Update Date:		1996/04/25 16:31:06
** Source File:		/share/dicom/cvs-depot/dcmtk/imagectn/libsrc/dbindex.cc,v
** CVS/RCS Revision:	1.3
** Status:		Exp
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include <stdio.h>
#ifdef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#ifdef HAVE_SYS_FILE_H
#include <sys/file.h>
#endif
#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif
#ifdef HAVE_SYS_STAT_H
#include <sys/stat.h>
#endif
#ifdef HAVE_UNISTD_H
#include <unistd.h>
#endif
#include <errno.h>

#include "imagedb.h"
#include "dbpriv.h"


/************
 *	Initializes addresses in an IdxRecord
 */

void 
DB_IdxInitRecord (IdxRecord *idx, int linksOnly) 
{
    int i ;
    i = 0 ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientBirthDate  ;
	idx -> param[i]. ValueLength = DA_MAX_LENGTH  ;
	idx -> PatientsBirthDate[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> PatientsBirthDate ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientSex  ;
	idx -> param[i]. ValueLength = CS_MAX_LENGTH  ;
	idx -> PatientsSex[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> PatientsSex ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientName  ;
	idx -> param[i]. ValueLength = PN_MAX_LENGTH  ;
	idx -> PatientsName[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> PatientsName ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientID  ;
	idx -> param[i]. ValueLength = LO_MAX_LENGTH  ;
	idx -> PatientID[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> PatientID ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientBirthTime  ;
	idx -> param[i]. ValueLength = TM_MAX_LENGTH  ;
	idx -> PatientsBirthTime[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> PatientsBirthTime ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_OtherPatientID  ;
	idx -> param[i]. ValueLength = LO_MAX_LENGTH  ;
	idx -> OtherPatientIDs[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> OtherPatientIDs ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_OtherPatientName  ;
	idx -> param[i]. ValueLength = PN_MAX_LENGTH  ;
	idx -> OtherPatientNames[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> OtherPatientNames ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_EthnicGroup  ;
	idx -> param[i]. ValueLength = SH_MAX_LENGTH  ;
	idx -> EthnicGroup[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> EthnicGroup ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_NumberOfPatientRelatedStudies  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH  ;
	idx -> NumberofPatientRelatedStudies[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *)idx -> NumberofPatientRelatedStudies ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_NumberOfPatientRelatedSeries  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH  ;
	idx -> NumberofPatientRelatedSeries[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> NumberofPatientRelatedSeries ;
    i++ ;
	
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_NumberOfPatientRelatedImages  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH  ;
	idx -> NumberofPatientRelatedImages[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> NumberofPatientRelatedImages ;
    i++ ;
				
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_StudyDate  ;
	idx -> param[i]. ValueLength = DA_MAX_LENGTH  ;
	idx -> StudyDate[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> StudyDate ;
    i++ ;
						
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_StudyTime  ;
	idx -> param[i]. ValueLength = TM_MAX_LENGTH  ;
	idx -> StudyTime[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> StudyTime ;
    i++ ;
						
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_StudyID  ;
	idx -> param[i]. ValueLength = CS_MAX_LENGTH  ;
	idx -> StudyID[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> StudyID ;
    i++ ;
				
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_AccessionNumber  ;
	idx -> param[i]. ValueLength = CS_MAX_LENGTH ;
	idx -> AccessionNumber[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> AccessionNumber ;
    i++ ;
								
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_ReferringPhysicianName  ;
	idx -> param[i]. ValueLength = PN_MAX_LENGTH ;
	idx -> ReferringPhysiciansName[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> ReferringPhysiciansName ;
    i++ ;
					
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_StudyDescription  ;
	idx -> param[i]. ValueLength = LO_MAX_LENGTH ;
	idx -> ProcedureDescription[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> ProcedureDescription ;
    i++ ;
					
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PhysicianReadingStudy  ;
	idx -> param[i]. ValueLength = PN_MAX_LENGTH ;
	idx -> AttendingPhysiciansName[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> AttendingPhysiciansName ;
    i++ ;
						
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_StudyInstanceUID  ;
	idx -> param[i]. ValueLength = UI_MAX_LENGTH ;
	idx -> StudyInstanceUID[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> StudyInstanceUID ;
    i++ ;
								
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_OtherStudyNumbers  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH ;
	idx -> OtherStudyNumbers[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> OtherStudyNumbers ;
    i++ ;
							
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_AdmittingDiagnosisDescription  ;
	idx -> param[i]. ValueLength = LO_MAX_LENGTH ;
	idx -> AdmittingDiagnosesDescription[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> AdmittingDiagnosesDescription ;
    i++ ;
				
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientAge  ;
	idx -> param[i]. ValueLength = AS_MAX_LENGTH ;
	idx -> PatientsAge[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> PatientsAge ;
    i++ ;
			
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientSize  ;
	idx -> param[i]. ValueLength = DS_MAX_LENGTH ;
	idx -> PatientsSize[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> PatientsSize ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_PatientWeight  ;
	idx -> param[i]. ValueLength = DS_MAX_LENGTH ;
	idx -> PatientsWeight[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> PatientsWeight ;
    i++ ;
			
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_Occupation  ;
	idx -> param[i]. ValueLength = SH_MAX_LENGTH ;
	idx -> Occupation[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> Occupation ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_NumberOfStudyRelatedSeries  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH ;
	idx -> NumberofStudyRelatedSeries[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> NumberofStudyRelatedSeries ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_NumberOfStudyRelatedImages  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH ;
	idx -> NumberofStudyRelatedImages[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> NumberofStudyRelatedImages ;
    i++ ;
			
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_SeriesNumber  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH ;
	idx -> SeriesNumber[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> SeriesNumber ;
    i++ ;
			
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_SeriesInstanceUID  ;
	idx -> param[i]. ValueLength = UI_MAX_LENGTH ;
	idx -> SeriesInstanceUID[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> SeriesInstanceUID ;
    i++ ;
			
    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_Modality  ;
	idx -> param[i]. ValueLength = CS_MAX_LENGTH ;
	idx -> ImageNumber[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> Modality ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_ImageNumber  ;
	idx -> param[i]. ValueLength = IS_MAX_LENGTH ;
	idx -> ImageNumber[0] = '\0' ;
    }
    idx -> param[i]. PValueField = (char *) idx -> ImageNumber ;
    i++ ;

    if (! linksOnly) {
	idx -> param[i]. XTag = DCM_SOPInstanceUID  ;
	idx -> param[i]. ValueLength = UI_MAX_LENGTH ;
	idx -> SOPInstanceUID[0] = '\0'	;
    }
    idx -> param[i]. PValueField = (char *) idx -> SOPInstanceUID ;
    i++ ;


}

/******************************
 *      Seek to a file position and do error checking
 *
 * Motivation:
 * We have had situations during demonstrations where size of the DB index file
 * has exploded.  It seems that a record is being written to a position
 * way past the end of file.
 * This seek function does some sanity error checking to try to identify 
 * the problem.
 */
long DB_lseek(int fildes, long offset, int whence)
{
    long pos;
    long curpos;
    long endpos;


    /* 
    ** we should not be seeking to an offset < 0
    */
    if (offset < 0) {
	fprintf(stderr, 
		"*** DB ALERT: attempt to seek before begining of file\n");
    }

    /* get the current position */
    curpos = lseek(fildes, 0, SEEK_CUR);
    if (curpos < 0) {
	perror("DB_lseek: cannot get current position");
	return curpos;
    }
    /* get the end of file position */
    endpos = lseek(fildes, 0, SEEK_END);
    if (endpos < 0) {
	perror("DB_lseek: cannot get end of file position");
	return endpos;
    }

    /* return to current position */
    curpos = lseek(fildes, curpos, SEEK_SET);
    if (curpos < 0) {
	perror("DB_lseek: cannot reset current position");
	return curpos;
    }

    /* do the requested seek */
    pos = lseek(fildes, offset, whence);
    if (pos < 0) {
	char msg[1024];
	sprintf(msg, "DB_lseek: cannot seek to %ld", offset);
	perror(msg);
	return pos;
    }

    /* 
    ** print an alert if we are seeking to far
    ** what is the limit? We don't expect the index file to be 
    ** larger than 32Mb
    */
    const long maxFileSize = 33554432;
    if (pos > maxFileSize) {
	fprintf(stderr, "*** DB ALERT: attempt to seek beyond %ld bytes\n",
		maxFileSize);
    }

    /* print an alert if we are seeking beyond the end of file.
     * ignore when file is empty
     */
    if ((endpos > 0) && (pos > endpos)) {
	fprintf(stderr, "*** DB ALERT: attempt to seek beyond end of file\n");
	fprintf(stderr, "              offset=%ld filesize=%ld\n", 
		offset, endpos);
    }

    return pos;
}

/******************************
 *	Read an Index record
 */

CONDITION DB_IdxRead (DB_Private_Handle *phandle, int idx, IdxRecord *idxRec)
{

    /*** Goto the right index in file
    **/
	
    DB_lseek (phandle -> pidx, (long) (SIZEOF_STUDYDESC + idx * SIZEOF_IDXRECORD), SEEK_SET) ; 

    /*** Read the record
    **/

    if (read (phandle -> pidx, (char *) idxRec, SIZEOF_IDXRECORD) != SIZEOF_IDXRECORD)
	return (DB_ERROR) ;

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ;	

    /*** Initialize record links
    **/

    DB_IdxInitRecord (idxRec, 1) ;
    return DB_NORMAL ;
}


/******************************
 *	Add an Index record
 *	Returns the index allocated for this record
 */

CONDITION DB_IdxAdd (DB_Private_Handle *phandle, int *idx, IdxRecord *idxRec)
{
    IdxRecord	rec ;
    CONDITION	cond ;

    /*** Find free place for the record
    *** A place is free if filename is empty
    **/

    *idx = 0 ;

    DB_lseek (phandle -> pidx, (long) SIZEOF_STUDYDESC, SEEK_SET) ; 
    while (read (phandle -> pidx, (char *) &rec, SIZEOF_IDXRECORD) == SIZEOF_IDXRECORD) {
	if (rec. filename [0] == '\0')
	    break ;
	(*idx)++ ;
    }

    /*** We have either found a free place or we are at the end of file. **/

	
    DB_lseek (phandle -> pidx, (long) (SIZEOF_STUDYDESC + (*idx) * SIZEOF_IDXRECORD), SEEK_SET) ; 

    if (write (phandle -> pidx, (char *) idxRec, SIZEOF_IDXRECORD) != SIZEOF_IDXRECORD)
	cond = DB_ERROR ;
    else
	cond = DB_NORMAL ;

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ;	

    return cond ;
}


/******************************
 *	Change the StudyDescRecord
 */

CONDITION DB_StudyDescChange (DB_Private_Handle *phandle, StudyDescRecord *pStudyDesc)
{
    CONDITION	cond ;

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ; 

    if (write (phandle -> pidx, (char *) pStudyDesc, SIZEOF_STUDYDESC) != SIZEOF_STUDYDESC)
	cond = DB_ERROR ;
    else
	cond = DB_NORMAL ;

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ;	

    return cond ;
}



/******************************
 *	Init an Index record loop
 */

CONDITION DB_IdxInitLoop (DB_Private_Handle *phandle, int *idx)
{
    DB_lseek (phandle -> pidx, SIZEOF_STUDYDESC, SEEK_SET) ; 
    *idx = -1 ;
    return DB_NORMAL ;
}

/******************************
 *	Get next Index record
 *	On return, idx is initialized with the index of the record read
 */

CONDITION DB_IdxGetNext (DB_Private_Handle *phandle, int *idx, IdxRecord *idxRec)
{

    (*idx)++ ;
    DB_lseek (phandle -> pidx, SIZEOF_STUDYDESC + (long)(*idx) * SIZEOF_IDXRECORD, SEEK_SET) ; 
    while (read (phandle -> pidx, (char *) idxRec, SIZEOF_IDXRECORD) == SIZEOF_IDXRECORD) {
	if (idxRec -> filename [0] != '\0') {
	    DB_IdxInitRecord (idxRec, 1) ;

	    return DB_NORMAL ;
	}
	(*idx)++ ;
    }

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ;	

    return DB_ERROR ;
}


/******************************
 *	Get next Index record
 *	On return, idx is initialized with the index of the record read
 */

CONDITION DB_GetStudyDesc (DB_Private_Handle *phandle, StudyDescRecord *pStudyDesc)
{

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ; 
    if ( read (phandle -> pidx, (char *) pStudyDesc, SIZEOF_STUDYDESC) == SIZEOF_STUDYDESC ) 
	return DB_NORMAL ;

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ;	

    return DB_ERROR ;
}


/******************************
 *	Remove an Index record
 *	Just put a record with filename == ""
 */

CONDITION DB_IdxRemove (DB_Private_Handle *phandle, int idx)
{
    IdxRecord 	rec ;
    CONDITION 	cond ;

    DB_lseek (phandle -> pidx, SIZEOF_STUDYDESC + (long)idx * SIZEOF_IDXRECORD, SEEK_SET) ;
    DB_IdxInitRecord (&rec, 0) ;

    rec. filename [0] = '\0' ;
    if (write (phandle -> pidx, (char *) &rec, SIZEOF_IDXRECORD) == SIZEOF_IDXRECORD) 
	cond = DB_NORMAL ;

    else 
	cond = DB_ERROR ;

    DB_lseek (phandle -> pidx, 0L, SEEK_SET) ;	

    return cond ;
}

/************************
 *	Dump an index file
 */

void	
DB_PrintIndexFile (char *storeArea)
{
    int i ;
    int j ;
    IdxRecord 		idxRec ;
    StudyDescRecord	*pStudyDesc;
    DB_Private_Handle	*phandle ;

    DB_createHandle (storeArea, -1, -1, (DB_Handle **) &phandle) ;

    pStudyDesc = (StudyDescRecord *)malloc (SIZEOF_STUDYDESC) ;
    if (pStudyDesc == NULL) {
	fprintf(stderr, "DB_PrintIndexFile: out of memory\n");
	return;
    }

    DB_lock(phandle, FALSE);

    DB_GetStudyDesc(phandle, pStudyDesc);

    for (i=0; i<phandle->maxStudiesAllowed; i++) {
	if (pStudyDesc[i].NumberofRegistratedImages != 0 ) {
	    printf("******************************************************\n");
	    printf("STUDY DESCRIPTOR: %d\n", i);
	    printf("  Study UID: %s\n", pStudyDesc[i].StudyInstanceUID);
	    printf("  StudySize: %ld\n", pStudyDesc[i].StudySize);
	    printf("  LastRecDate: %f\n", pStudyDesc[i].LastRecordedDate);
	    printf("  NumOfImages: %d\n", 
		   pStudyDesc[i].NumberofRegistratedImages);
	}
    }

    DB_IdxInitLoop (phandle, &j) ;
    while (1) {
	if (DB_IdxGetNext (phandle, &j, &idxRec) != DB_NORMAL)
	    break ;

	printf("*******************************************************\n") ;
	printf("RECORD NUMBER %d\n",j) ;
	printf("  Filename: %s\n", idxRec. filename) ;
	printf("  ImageSize: %d\n", idxRec. ImageSize) ;
	printf("  RecordedDate: %f\n", idxRec. RecordedDate) ;
	for (i = 0 ; i < NBPARAMETERS ; i++) {	/* new definition */
	    DB_SmallDcmElmt *se = idxRec.param + i;
	    char* value = "";
	    if (se->PValueField != NULL) value = se->PValueField;
	    DcmTag tag(se->XTag);
	    printf("    %s: \"%s\"\n", tag.getTagName(), value);
	}
    }
    printf("*******************************************************\n") ;
    printf("RECORDS IN THIS INDEXFILE: %d\n",j) ;

    DB_unlock(phandle);
	
    DB_destroyHandle ((DB_Handle **) &phandle) ;
}

/*
** CVS Log
** dbindex.cc,v
** Revision 1.3  1996/04/25 16:31:06  hewett
** Revised lseek checking to allow for zero length file case.
**
** Revision 1.2  1996/04/22 11:18:56  hewett
** Added sanity checking to calls of lseek().
**
** Revision 1.1.1.1  1996/03/28 19:24:59  hewett
** Oldenburg Image CTN Software ported to use the dcmdata C++ toolkit.
**
**
*/
