/*
**  Copyright (C) 1993/1994, OFFIS, Oldenburg University and CERIUM
**  
**  This software and supporting documentation were
**  developed by 
**  
**    Institut OFFIS
**    Bereich Kommunikationssysteme
**    Westerstr. 10-12
**    26121 Oldenburg, Germany
**    
**    Fachbereich Informatik
**    Abteilung Prozessinformatik
**    Carl von Ossietzky Universitaet Oldenburg 
**    Ammerlaender Heerstr. 114-118
**    26111 Oldenburg, Germany
**    
**    CERIUM
**    Laboratoire SIM
**    Faculte de Medecine
**    2 Avenue du Pr. Leon Bernard
**    35043 Rennes Cedex, France
**  
**  for CEN/TC251/WG4 as a contribution to the Radiological 
**  Society of North America (RSNA) 1993 Digital Imaging and 
**  Communications in Medicine (DICOM) Demonstration.
**  
**  THIS SOFTWARE IS MADE AVAILABLE, AS IS, AND NEITHER OFFIS,
**  OLDENBURG UNIVERSITY NOR CERIUM MAKE ANY WARRANTY REGARDING 
**  THE SOFTWARE, ITS PERFORMANCE, ITS MERCHANTABILITY OR 
**  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER 
**  DISEASES OR ITS CONFORMITY TO ANY SPECIFICATION.  THE 
**  ENTIRE RISK AS TO QUALITY AND PERFORMANCE OF THE SOFTWARE   
**  IS WITH THE USER. 
**  
**  Copyright of the software and supporting documentation
**  is, unless otherwise stated, jointly owned by OFFIS,
**  Oldenburg University and CERIUM and free access is hereby
**  granted as a license to use this software, copy this
**  software and prepare derivative works based upon this
**  software. However, any distribution of this software
**  source code or supporting documentation or derivative
**  works (source code and supporting documentation) must
**  include the three paragraphs of this copyright notice. 
** 
*/

/*
** 
** Author:  Andrew Hewett		Created: 23-03-93 
**	     OFFIS, Oldenburg, Germany
**
** Rewritten
** 
** Program: dbregimg 
** 
** Purpose: 	
** This test program registers image files in the image database.
** 
**
** Last Update:		hewett
** Update Date:		1996/04/29 10:47:43
** Source File:		/share/dicom/cvs-depot/dcmtk/imagectn/tests/dbregimg.cc,v
** CVS/RCS Revision:	1.3
** Status:		Exp
**
** CVS/RCS Log at end of file
*/

#include "osconfig.h"    /* make sure OS specific configuration is included first */

#include <stdio.h>
#ifndef HAVE_STDLIB_H
#include <stdlib.h>
#endif
#include <string.h>

#include "imagedb.h"
#include "diutil.h"
#include "dcdebug.h"
#include "dcompat.h"
#include "dcdict.h"

static void 
usage (char *progname)
{
    fprintf(stderr, "usage: %s [-d n] [-v] [-p] storageArea [imageFile...] \n", 
	    progname) ;
    fprintf(stderr, "Register a dicom image file in an image database index file\n") ;
    fprintf(stderr, "Options:\n");
    fprintf(stderr, "  -d n  set debug level to n (an integer)\n");
    fprintf(stderr, "  -v     be verbose\n");
    fprintf(stderr, "  -p     list contents of database index file\n");
    exit (1);
}


int main (int argc, char *argv[])
{
    DB_Handle *hdl = NULL;
    char sclass [120] ;
    char sinst  [120] ;
    char fname  [120] ;
    DB_Status status; 
    int i, j;
    int debugLevel = 0;
    BOOL verbose = FALSE;
    BOOL printIndexFile = FALSE;

    /* parse program arguments */
    for (i = 1; i < argc && argv[i][0] == '-'; i++) {
	switch (argv[i][1]) {
	case 'd':
	    if (((i + 1) < argc) && (argv[i + 1][0] != '-') &&
		(sscanf(argv[i + 1], "%d", &debugLevel) == 1)) {
		i++;		
	    } else {
		debugLevel = 1;
	    }
	    break;
	case 'v':
	    verbose = TRUE;
	    break;
	case 'p':
	    printIndexFile = TRUE;
	    break;
	default:
	    usage(argv[0]);
	}
    }

    if ((argc - i) < 1) {
	usage(argv[0]);
    }

    for (j=i; j<argc; j++) {
	if (access(argv[j], R_OK) < 0) {
	    fprintf(stderr, "cannot access: %s\n", argv[j]);
	    usage(argv[0]);
	}
    }

    /* make sure data dictionary is loaded */
    if (dcmDataDict.numberOfEntries() == 0) {
	fprintf(stderr, "Warning: no data dictionary loaded, check environment variable: %s\n",
		DCM_DICT_ENVIRONMENT_VARIABLE);
    }

    DB_setDebugLevel(debugLevel);	/* DB debugging */
    SetDebugLevel((debugLevel));	/* dcmdata debugging */

    char* dbLocation = argv[i];
    i++;

    DB_enableQuotaSystem(FALSE); /* disable deletion of images */

    DB_createHandle (dbLocation, DB_UpperMaxStudies, 
		     DB_UpperMaxBytesPerStudy, &hdl) ;

    for (j=i; j<argc; j++) {

	if (verbose) {
	    printf("Registering: %s\n", argv[j]);
	}
	BOOL ok = DU_findSOPClassAndInstanceInFile(argv[j], sclass, sinst);

	if (!ok) {
	    fprintf(stderr, "%s: cannot load dicom file: %s\n",
		    argv[0], argv[j]);
	} else {

#ifdef DEBUG
	    if (DB_getDebugLevel() > 0) {
		/*** Test what filename is recommended by DB_Module	**/
		DB_makeNewStoreFileName (hdl, sclass, sinst, fname) ;
		printf ("DB_Module recommends %s for filename\n", fname) ;
	    }
#endif
	    DB_storeRequest (hdl, sclass, sinst, argv[j], &status) ;
	}
    }

    DB_destroyHandle (&hdl) ;	

    if (printIndexFile) {
	printf("-- DB Index File --\n");
	DB_PrintIndexFile(dbLocation);
    }

    return 0;

}


