
/*
 *
 *  Copyright (C) 1996, OFFIS
 *
 *  This software and supporting documentation were developed by
 *
 *    Kuratorium OFFIS e.V.
 *    Forschungsbereich 2: Kommunikationssysteme
 *    Escherweg 2
 *    D-26121 Oldenburg, Germany
 *
 *  for CEN/TC251/WG4 as a contribution to the Computer Assisted Radiology
 *  (CAR) 1996 DICOM Demonstration.
 *
 *  THIS SOFTWARE IS MADE AVAILABLE,  AS IS,  AND OFFIS MAKES NO  WARRANTY
 *  REGARDING  THE  SOFTWARE,  ITS  PERFORMANCE,  ITS  MERCHANTABILITY  OR
 *  FITNESS FOR ANY PARTICULAR USE, FREEDOM FROM ANY COMPUTER DISEASES  OR
 *  ITS CONFORMITY TO ANY SPECIFICATION. THE ENTIRE RISK AS TO QUALITY AND
 *  PERFORMANCE OF THE SOFTWARE IS WITH THE USER.
 *
 *  Copyright of the software  and  supporting  documentation  is,  unless
 *  otherwise stated, owned by OFFIS, and free access is hereby granted as
 *  a license to  use  this  software,  copy  this  software  and  prepare
 *  derivative works based upon this software.  However, any  distribution
 *  of this software source code or supporting documentation or derivative
 *  works  (source code and  supporting documentation)  must  include  the
 *  three paragraphs of this copyright notice.
 *
 *
 * Module: DICOM Basic Worklist Management CTN, WWW Component
 *
 * Author: Marco Eichelberg
 *
 * Purpose:
 *   This application creates binary coded "struct flock"  fields for  use
 *   within the "fcntl" function of a perl script. The output of this file
 *   is system dependent, therefore it should be called dynamically  (e.g.
 *   upon startup of the perl script). This  will  prevent  problems  with
 *   systems supporting fat binaries (e.g. NeXTStep).
 *
 *   Usage example (from perl):
 *     $fcntldata=`./preplock`;
 *     eval $fcntldata;
 *   This will create four global perl variables:
 *     $const_F_SETLKW      (2nd fnctl parameter for locking files)
 *     $const_RDLCK_STRUCT  (3rd   "      "       "  read lock)
 *     $const_WRLCK_STRUCT  ( "    "      "       "  write lock)
 *     $const_UNLCK_STRUCT  ( "    "      "       "  unlock)
 *
 *   A perl fcntl call using these (packed binary) values will lock/unlock a
 *   whole file, which must be a successfully opened perl  file  descriptor.
 *   See perl man page for details. 
 *
 * Last Update:      meichel
 * Update Date:      1996/05/31 11:43:34
 * Source File:      /share/dicom/cvs-depot/dcmtk/wlistctn/wwwapps/preplock.c,v
 * CVS/RCS Revision: 1.1
 * Status:           Exp
 *
 * CVS/RCS Log
 *   preplock.c,v
 *   Revision 1.1  1996/05/31 11:43:34  meichel
 *   Initial release of the C/C++ code for the Worklist CTN WWW/CGI application.
 *
 *
 */

#include "osconfig.h"

#include <stdio.h>
#include <fcntl.h>

int main()
{
  struct flock lockdata;
  unsigned int i;
  unsigned char *c;

  /* first, we initialize the data */  
  c = (unsigned char *)(&lockdata);
  for (i=0; i<sizeof(struct flock); i++) c[i]=0;

  lockdata.l_whence=0;
  lockdata.l_start=0;
  lockdata.l_len=0;

  /* now we create the output */
  printf("$const_F_SETLKW = %ld;\n",(long)F_SETLKW);

  lockdata.l_type = F_RDLCK;
  printf("$const_RDLCK_STRUCT = pack(\"");
  for (i=0; i<sizeof(struct flock); i++) printf("c");
  printf("\"");
  for (i=0; i<sizeof(struct flock); i++) printf(",%d",(int)(c[i]));
  printf(");\n");
  
  lockdata.l_type = F_WRLCK;
  printf("$const_WRLCK_STRUCT = pack(\"");
  for (i=0; i<sizeof(struct flock); i++) printf("c");
  printf("\"");
  for (i=0; i<sizeof(struct flock); i++) printf(",%d",(int)(c[i]));
  printf(");\n");
  
  lockdata.l_type = F_UNLCK;
  printf("$const_UNLCK_STRUCT = pack(\"");
  for (i=0; i<sizeof(struct flock); i++) printf("c");
  printf("\"");
  for (i=0; i<sizeof(struct flock); i++) printf(",%d",(int)(c[i]));
  printf(");\n");

  return 0;  
}

